import org.scalatest._

class Dice34Spec extends FlatSpec with Matchers {

  val Dices = new Dices(3,4)

  "There" should "be three faces" in {
    Dices.Face.values.count({_=>true}) should be (3)
    // assert(Dices.Face.values.count({_ => true})==3)
  }

  it should "be the total number of dices" in {
    val count = Dices.count()
    (new Dices.Dices).count({_=>true}) should be (count)
  }

  it should "be the total number of dices minus 3" in {
    val count = Dices.count()
    val dices = new Dices.Dices
    for (i <- 1 to 3) dices.next
    dices.count({_=>true}) should be (count-3)
  }

  "Dices" should "be compared structurally" in {
     val a = new Dices.Dice(Vector(1,1,1,1))
     val b = new Dices.Dice(Vector(1,1,1,1))
     assert (a == b)
  }

  "Iterators" should "be generated new" in {
    val dices1 = new Dices.Dices
    val dices2 = new Dices.Dices
    val d1 = dices1.next
    val d2 = dices2.next
    assert (d1.compare(d2)==0) // TODO meaning of == on Dice
    // d1 should be (d2)
  }

  it should "generate (three) distinct elements" in {
    val dices = new Dices.Dices
    val d1 = dices.next
    val d2 = dices.next
    val d3 = dices.next
    d1 should not be (d2)
    d2 should not be (d3)
    d3 should not be (d1)
  }

  it should "generate an increasing sequence" in {
     def test(prev:Dices.Dice,it:Iterator[Dices.Dice]) = {
       val more = it.next
       assert(more>prev)
     }
     val d = new Dices.Dices
     test(d.next,d)
  }

  "Functional iterator" should
    "enumerate as many elements as imperative one" in
  {
    // Dices.iter(Dices.Face.min,Dices.nb_faces).foreach(println)
    assert((new Dices.Dices).count({_ => true}) ==
      Dices.iter(Dices.Face.min,Dices.nb_faces).count({_ => true}))
  }

  "Being better" should "not hold both on (#3,#7) and (#7,#3)" in {
    val s = (new Dices.Dices).toIndexedSeq
    val d = s(3)
    val e = s(7)
    assert (!(d >> e && e >> d))
  }

  it should "be irreflexive" in {
    for {
      i <- new Dices.Dices
      j <- new Dices.Dices(i)
    } {
      assert (!(i>>i))
    }
  }

  it should "be antisymmetric" in {
    for {
      i <- new Dices.Dices
      j <- new Dices.Dices(i)
    } {
      assert (!(i>>j && j>>i))
    }
  }

}
